//  This file was generated with SpriteHelper
//  http://www.spritehelper.org
//
//  SpriteHelperLoader.mm
//  Created by Bogdan Vladu
//  Copyright 2011 Bogdan Vladu. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
//
//  This software is provided 'as-is', without any express or implied
//  warranty.  In no event will the authors be held liable for any damages
//  arising from the use of this software.
//  Permission is granted to anyone to use this software for any purpose,
//  including commercial applications, and to alter it and redistribute it
//  freely, subject to the following restrictions:
//  The origin of this software must not be misrepresented; you must not
//  claim that you wrote the original software. If you use this software
//  in a product, an acknowledgment in the product documentation would be
//  appreciated but is not required.
//  Altered source versions must be plainly marked as such, and must not be
//  misrepresented as being the original software.
//  This notice may not be removed or altered from any source distribution.
//  By "software" the author refers to this code file and not the application 
//  that was used to generate this file.
//
////////////////////////////////////////////////////////////////////////////////

#include "SpriteHelperLoader.h"
#include "iostream"
#include "assert.h"
#include "fstream"
#include <sstream>
#include <map>
#include "platform/CCNS.h"
////////////////////////////////////////////////////////////////////////////////
////////////////SPECIAL CLASSES FOR COCOS2D-X TO READ FROM A PLIST FILE/////////
////////////////////////////////////////////////////////////////////////////////
class SpriteInfo
{
public:
	SpriteInfo(){};
	~SpriteInfo(){properties.clear(); fixtures.clear();};
	
	LHDictionary properties;
	std::vector<std::vector<std::string> > fixtures;
};
////////////////////////////////////////////////////////////////////////////////
class AnimInfo
{
public:
    AnimInfo(){}
    ~AnimInfo(){properties.clear(); frames.clear();}
    
    LHDictionary properties;
    std::vector<std::string> frames;
};
////////////////////////////////////////////////////////////////////////////////
/// converts degrees to radians
#define DEGREES_TO_RADIANS(__ANGLE__) ((__ANGLE__) / 180.0f * (float)M_PI)
/// converts radians to degrees
#define RADIANS_TO_DEGREES(__ANGLE__) ((__ANGLE__) / (float)M_PI * 180.0f)

std::string concatenate(std::string const& str, int i)
{
    stringstream s;
    s << str << i;
    return s.str();
}
////////////////////////////////////////////////////////////////////////////////
SpriteHelperLoader::SpriteHelperLoader(const std::string& levelFile)
{
	loadSpriteHelperSceneFile(levelFile);
}
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::assertMessage(bool value, const std::string message)
{
	if(value == false)
	{
		CCLog(message.c_str());
	}
	
	assert(value);
}
////////////////////////////////////////////////////////////////////////////////

CCSprite* SpriteHelperLoader::spriteWithUniqueName(const std::string &name, 
                                                   const CCPoint& position, 
                                                   CCLayer* layer)
{
    for(size_t i = 0; i < lhSprites.size(); ++i)
	{
		SpriteInfo* sprite = lhSprites[i];
        
		if(sprite->properties["Name"] == name)
		{
			CCSprite* ccsprite = spriteFromMap(sprite->properties);
            ccsprite->setPosition(position);
            
			if(0 != ccsprite && 0 != layer)
				layer->addChild(ccsprite, atoi(sprite->properties["ZOrder"].c_str()));
			
			return ccsprite;
		}
	}
	return NULL;
}

////////////////////////////////////////////////////////////////////////////////
b2Body* SpriteHelperLoader::bodyWithUniqueName(const std::string& uniqueName, 
                                               const CCPoint& position,
                                               CCLayer* layer,
                                               b2World* world)
{
	for(size_t i = 0; i < lhSprites.size(); ++i)
	{
		SpriteInfo* sprite = lhSprites[i];
		
		if(sprite->properties["Name"] == uniqueName)
		{
			CCSprite* ccsprite = spriteFromMap(sprite->properties);
            ccsprite->setPosition(position);
			if(0 != ccsprite)
			{
                if(0 != layer)
                    layer->addChild(ccsprite, atoi(sprite->properties["ZOrder"].c_str()));
				
				return b2BodyFromMap(sprite->properties, 
									 sprite->fixtures,
									 ccsprite,
									 world);
			}
		}
	}
	
	return NULL;
}
////////////////////////////////////////////////////////////////////////////////
CCSprite* SpriteHelperLoader::spriteInBatchWithUniqueName(const std::string &name, 
                                                          const CCPoint& position, 
                                                          CCLayer* layer)
{
    for(size_t i = 0; i < lhSprites.size(); ++i)
	{
		SpriteInfo* sprite = lhSprites[i];
        
		if(sprite->properties["Name"] == name)
		{
			CCSprite* ccsprite = spriteFromMap(sprite->properties);
            ccsprite->setPosition(position);
            
            CCSpriteBatchNode* node = NULL;
            if(batchInfo.end() == batchInfo.find(sprite->properties["Image"]))
            {
                node = CCSpriteBatchNode::batchNodeWithFile(sprite->properties["Image"].c_str());
                if(0 != layer)
                    layer->addChild(node, batchOrder);
                
                node->addChild(ccsprite, atoi(sprite->properties["ZOrder"].c_str()));
                batchInfo[sprite->properties["Image"]] = node;
            }
            else
            {
                node = batchInfo[sprite->properties["Image"]];
                node->addChild(ccsprite, atoi(sprite->properties["ZOrder"].c_str()));
            }
			return ccsprite;
		}
	}
	return NULL;
}

b2Body* SpriteHelperLoader::bodyInBatchWithUniqueName(const std::string& uniqueName, 
                                                      const CCPoint& position,
                                                      CCLayer* layer,
                                                      b2World* world)
{
	for(size_t i = 0; i < lhSprites.size(); ++i)
	{
		SpriteInfo* sprite = lhSprites[i];
		
		if(sprite->properties["Name"] == uniqueName)
		{
			CCSprite* ccsprite = spriteFromMap(sprite->properties);
            ccsprite->setPosition(position);
			if(0 != ccsprite)
			{
                CCSpriteBatchNode* node = NULL;
                if(batchInfo.end() == batchInfo.find(sprite->properties["Image"]))
                {
                    node = CCSpriteBatchNode::batchNodeWithFile(sprite->properties["Image"].c_str());
                    if(0 != layer)
                        layer->addChild(node, batchOrder);
                    
                    node->addChild(ccsprite, atoi(sprite->properties["ZOrder"].c_str()));
                    batchInfo[sprite->properties["Image"]] = node;
                }
                else
                {
                    node = batchInfo[sprite->properties["Image"]];
                    node->addChild(ccsprite, atoi(sprite->properties["ZOrder"].c_str()));
                }
				
				return b2BodyFromMap(sprite->properties, 
									 sprite->fixtures,
									 ccsprite,
									 world);
			}
		}
	}
	
	return NULL;
    
}
////////////////////////////////////////////////////////////////////////////////
CCAction* SpriteHelperLoader::runAnimationWithUniqueName(const std::string& animName, 
                                                         CCSprite* spr,
                                                         SEL_CallFuncND notifSEL,
                                                         SelectorProtocol* notifObj)
{
    if(spr->isUsesBatchNode())
    {
        printf("SpriteHelper WARNING: Can't create animation on sprites that uses batch nodes. Please use spriteWithUniqueName or bodyWithUniqueName to create the sprite");
        return 0;
    }          
    
    if(shAnims.end() == shAnims.find(animName))
        return NULL;
    
    AnimInfo* animInfo = shAnims[animName];
    if(NULL == animInfo)
        return NULL;
    
    
    bool loop = atoi(animInfo->properties["Loop"].c_str());
    int repetitions = atoi(animInfo->properties["Repetitions"].c_str());
    float speed = atof(animInfo->properties["Speed"].c_str());
    bool startAtLaunch = atoi(animInfo->properties["StartAtLaunch"].c_str());
    
    if(animInfo->frames.size() == 0)
        return 0;
    
    LHDictionary* spriteProperties = propertyForName(animInfo->frames[0]);
    
    if(0 != spriteProperties)
    {                
        CCTexture2D* tex = CCTextureCache::sharedTextureCache()->addImage((*spriteProperties)["Image"].c_str());
        spr->setTexture(tex);
        
        CCSpriteFrame* frame1 = CCSpriteFrame::frameWithTexture(spr->getTexture(), 
                                                                CCRectFromString((*spriteProperties)["Frame"].c_str()));
        
        CCSpriteFrameCache::sharedSpriteFrameCache()->addSpriteFrame(frame1, animInfo->frames[0].c_str());
        
        CCMutableArray<CCSpriteFrame*> frames;
        frames.addObject(frame1);
        
        for(int i = 1; i < animInfo->frames.size(); ++i)
        {
            LHDictionary* frameSprite = propertyForName(animInfo->frames[i]);
            
            CCSpriteFrame* frame = CCSpriteFrame::frameWithTexture(spr->getTexture(), CCRectFromString((*frameSprite)["Frame"].c_str()));
            
            CCSpriteFrameCache::sharedSpriteFrameCache()->addSpriteFrame(frame, animInfo->frames[i].c_str());
            
            frames.addObject(frame);
        }
        
        CCAnimation *anim = CCAnimation::animationWithFrames(&frames, speed);
        
        CCAction* animAct; 
        
        if(loop == false){
            if(0 != notifObj && 0 != notifSEL){
                CCCallFuncND* notifAction = CCCallFuncND::actionWithTarget(notifObj, 
                                                                             notifSEL, 
                                                                             (void*)spr);
            
                animAct = CCSequence::actionOneTwo(CCAnimate::actionWithAnimation(anim, false), notifAction);
            }
            else{
                animAct = CCRepeat::actionWithAction(CCAnimate::actionWithAnimation(anim, false), repetitions);
            }
        }
        else{
            animAct = CCRepeatForever::actionWithAction(CCAnimate::actionWithAnimation(anim, false));
        }
        
        if(startAtLaunch)
            spr->runAction(animAct);
        
        return animAct;
    }    
    return 0;
}
////////////////////////////////////////////////////////////////////////////////
CCAction* SpriteHelperLoader::runAnimationWithUniqueName(const std::string& animName, 
                                                         CCSprite* spr)
{
    return runAnimationWithUniqueName(animName, spr, 0, 0);
}
////////////////////////////////////////////////////////////////////////////////
CCAction* SpriteHelperLoader::runAnimationWithUniqueName(const std::string& animName, 
                                                         const b2Body* body)
{    
    return runAnimationWithUniqueName(animName, body, 0, 0);
}
////////////////////////////////////////////////////////////////////////////////
CCAction* SpriteHelperLoader::runAnimationWithUniqueName(const std::string& animName, 
                                                         const b2Body* body,
                                                         SEL_CallFuncND notifSEL,
                                                         SelectorProtocol* notifObj)
{
    if(body == 0)
        return 0;
    
    CCSprite* sprite = (CCSprite*)body->GetUserData();
    return runAnimationWithUniqueName(animName, sprite, notifSEL, notifObj);    
}
////////////////////////////////////////////////////////////////////////////////
SpriteHelperLoader::~SpriteHelperLoader()
{		
	for(int i = 0; i < (int)lhSprites.size(); ++i){
		delete lhSprites[i];
	}
	lhSprites.clear();
    
    std::map<std::string, AnimInfo*>::iterator it;
    for ( it=shAnims.begin() ; it != shAnims.end(); it++ )
        delete (*it).second;
	shAnims.clear();
    
    batchInfo.clear(); //deleting the batch nodes is handled by Cocos2d-X
}
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
///////////////////////////PRIVATE METHODS//////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
LHDictionary* SpriteHelperLoader::propertyForName(const std::string& name)
{
    for(size_t i = 0; i < lhSprites.size(); ++i)
	{
		SpriteInfo* sprite = lhSprites[i];
        
		if(sprite->properties["Name"] == name)
		{
            return &lhSprites[i]->properties;
        }
    }
    return 0;
}
////////////////////////////////////////////////////////////////////////////////
SpriteInfo* SpriteHelperLoader::spriteInfoForName(const std::string& name)
{
    for(size_t i = 0; i < lhSprites.size(); ++i)
	{
		SpriteInfo* sprite = lhSprites[i];
        
		if(sprite->properties["Name"] == name)
		{
            return sprite;
        }
    }
    return 0;
}
////////////////////////////////////////////////////////////////////////////////
CCSprite* SpriteHelperLoader::spriteFromMap(LHDictionary &spriteMap)
{
	CCSprite *ccsprite = CCSprite::spriteWithFile(spriteMap["Image"].c_str(), 
												  cocos2d::CCRectFromString(spriteMap["Frame"].c_str()));
	setSpritePropertiesFromMap(ccsprite, spriteMap);
												  
	return ccsprite;
}
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::setSpritePropertiesFromMap(CCSprite* ccsprite, 
												   LHDictionary &spriteMap)
{
	//convert position from LH to Cocos2d coordinates
	ccsprite->setOpacity((GLubyte)(255*atof(spriteMap["Opacity"].c_str())));
	CCRect color = cocos2d::CCRectFromString(spriteMap["Color"].c_str());
	ccsprite->setColor(ccc3(255*color.origin.x, 255*color.origin.y, 255*color.size.width));
	CCPoint scale = cocos2d::CCPointFromString(spriteMap["Scale"].c_str());
	ccsprite->setScaleX(scale.x);
	ccsprite->setScaleY(scale.y);
}
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::setFixtureDefPropertiesFromMap(std::map<std::string, std::string>& spritePhysic,
													   b2FixtureDef*shapeDef)
{
	shapeDef->density = atof(spritePhysic["Density"].c_str());
	shapeDef->friction = atof(spritePhysic["Friction"].c_str());
	shapeDef->restitution = atof(spritePhysic["Restitution"].c_str());
	shapeDef->isSensor = (bool)atoi(spritePhysic["IsSenzor"].c_str());
	
	shapeDef->filter.categoryBits = atoi(spritePhysic["Category"].c_str());
	shapeDef->filter.maskBits = atoi(spritePhysic["Mask"].c_str());
	shapeDef->filter.groupIndex = atoi(spritePhysic["Group"].c_str());
}
////////////////////////////////////////////////////////////////////////////////
b2Body* SpriteHelperLoader::b2BodyFromMap(LHDictionary& properties,
										 std::vector<std::vector<std::string> >&fixtures,
										 CCSprite* ccsprite,
										 b2World* world)
{
	b2BodyDef bodyDef;	
	
	b2Vec2 position = b2Vec2(ccsprite->getPosition().x/PTM_RATIO,ccsprite->getPosition().y/PTM_RATIO);
	
    bodyDef.type = (b2BodyType)atoi(properties["PhysicType"].c_str());
	bodyDef.position = position;
	
	bodyDef.userData = ccsprite;
	b2Body* body = world->CreateBody(&bodyDef);
	
	body->SetFixedRotation((bool)atoi(properties["IsFixedRot"].c_str()));
	
	CCPoint scale = CCPointFromString(properties["Scale"].c_str()); 
    CCRect frame = CCRectFromString(properties["Frame"].c_str());
    CCSize size = frame.size;
    
    int shapeOffsetX = atoi(properties["ShapePositionOffsetX"].c_str());
    int shapeOffsetY = atoi(properties["ShapePositionOffsetY"].c_str());
    
    float linearVelocityX = atof(properties["LinearVelocityX"].c_str());
    float linearVelocityY = atof(properties["LinearVelocityX"].c_str());
    
    float linearDamping = atof(properties["LinearDamping"].c_str());
    float angularVelocity = atof(properties["AngularVelocity"].c_str());
    float angularDamping = atof(properties["AngularDamping"].c_str());
    
    bool isBullet = atoi(properties["IsBullet"].c_str());
    bool canSleep = atoi(properties["CanSleep"].c_str());
    
    body->SetLinearVelocity(b2Vec2(linearVelocityX, linearVelocityY));
    body->SetAngularVelocity(angularVelocity);
    body->SetLinearDamping(linearDamping);
    body->SetAngularDamping(angularDamping);
    body->SetBullet(isBullet);
    body->SetSleepingAllowed(canSleep);
    
	if(fixtures.size() == 0)
	{
        int shapeBorderX = atoi(properties["ShapeBorderW"].c_str());
        int shapeBorderY = atoi(properties["ShapeBorderH"].c_str());
        
		b2PolygonShape shape;
		b2FixtureDef fixture;
		b2CircleShape circle;
		setFixtureDefPropertiesFromMap(properties, &fixture);
		
		if((bool)atoi(properties["IsCircle"].c_str()))
		{
			circle.m_radius = (size.width*scale.x - shapeBorderX)/2/PTM_RATIO;
            circle.m_p.Set(shapeOffsetX/2.0f/PTM_RATIO, -shapeOffsetY/2.0f/PTM_RATIO);
			fixture.shape = &circle;
            body->CreateFixture(&fixture);
		}
		else
		{
            int vsize = 4;
			b2Vec2 *verts = new b2Vec2[vsize];
			b2PolygonShape shape;
			
            verts[0].x = ( (-1* size.width + shapeBorderX/2.0f)*scale.x/2.0f +shapeOffsetX/2.0f)/PTM_RATIO;
            verts[0].y = ( (-1* size.height + shapeBorderY/2.0f)*scale.y/2.0f-shapeOffsetY/2.0f)/PTM_RATIO;
            
            verts[1].x = ( (+ size.width - shapeBorderX/2.0f)*scale.x/2.0f+shapeOffsetX/2.0f)/PTM_RATIO;
            verts[1].y = ( (-1* size.height + shapeBorderY/2.0f)*scale.y/2.0f-shapeOffsetY/2.0f)/PTM_RATIO;
            
            verts[2].x = ( (+ size.width - shapeBorderX/2.0f)*scale.x/2.0f+shapeOffsetX/2.0f)/PTM_RATIO;
            verts[2].y = ( (+ size.height - shapeBorderY/2.0f)*scale.y/2.0f-shapeOffsetY/2.0f)/PTM_RATIO;
            
            verts[3].x = ( (-1* size.width + shapeBorderX/2.0f)*scale.x/2.0f+shapeOffsetX/2.0f)/PTM_RATIO;
            verts[3].y = ( (+ size.height - shapeBorderY/2.0f)*scale.y/2.0f-shapeOffsetY/2.0f)/PTM_RATIO;
            
            
			shape.Set(verts, vsize);		
            
			fixture.shape = &shape;
            body->CreateFixture(&fixture);
            delete verts;
		}
	}
	else
	{
		for(size_t i = 0; i < fixtures.size(); ++i)
		{
			std::vector<std::string>& curFixture(fixtures[i]);
			int size = (int)curFixture.size();
			b2Vec2 *verts = new b2Vec2[size];
			b2PolygonShape shape;
			int k = 0;
			
			for(int j = 0; j < size; ++j)
			{ 
				std::string pointStr = curFixture[j];
				CCPoint point = CCPointFromString(pointStr.c_str());
				verts[k] = b2Vec2((point.x*(scale.x)+shapeOffsetX/2.0f)/PTM_RATIO, 
								  (point.y*(scale.y)-shapeOffsetY/2.0f)/PTM_RATIO);
				++k;
			}
			shape.Set(verts, size);		
			b2FixtureDef fixture;
			setFixtureDefPropertiesFromMap(properties, &fixture);
			fixture.shape = &shape;
			body->CreateFixture(&fixture);
			delete verts;
		}
	}
	
	return body;	
}
////////////////////////////////////////////////////////////////////////////////

SpriteInfo currentSpriteInfo;
bool readSprites = false;

std::vector<std::vector<std::string> >  currentPolygonInfo;
std::vector<std::string> currentFixtureInfo;
bool readFixture = false;

AnimInfo currentAnimInfo;
std::vector<AnimInfo*> allAnims;
bool readAnims = false;
std::vector<std::string> currentAnimFrames;
bool readFrames = false;

std::string currentKey;

int dict = 0;
int array = 0;
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::key(const std::string& arg)
{
	currentKey = arg;
	if(std::string::npos != arg.find("SPRITES_INFO")){
		readSprites = true;
	}
	else if(std::string::npos != arg.find("Fixtures")){
		readFixture = true;
	}
    else if(std::string::npos != arg.find("ANIMATION_INFO")){
		readAnims = true;
	}
    else if(std::string::npos != arg.find("Frames")){
		readFrames = true;
	}
}
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::pushValue(const std::string& arg)
{
	if(readSprites){
		currentSpriteInfo.properties[currentKey] = arg;
	}
	if(readFixture){
		currentFixtureInfo.push_back(arg);
	}
    if(readAnims){
        if(!readFrames)
            currentAnimInfo.properties[currentKey] = arg;
    }
    if(readFrames){
        currentAnimFrames.push_back(arg);
    }
}
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::arrayBegin(){
   
	array += 1;
}
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::noArray(){

    readFixture = false;
    currentFixtureInfo.clear();
    currentPolygonInfo.clear();
    
    readFrames = false;
    currentAnimFrames.clear();
}
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::arrayEnd(){
	array -= 1;

	if(0 == array){
		if(readSprites){
			readSprites = false;
		}		
	}

	if(readFixture){
		if(2 == array){
			currentPolygonInfo.push_back(currentFixtureInfo);
			currentFixtureInfo.clear();
		}
		else if(1 == array){
			currentSpriteInfo.fixtures = currentPolygonInfo;
			readFixture = false;
			currentPolygonInfo.clear();
		}
	}
    
    if(readFrames){
        if(1 == array){
            currentAnimInfo.frames = currentAnimFrames;
            readFrames = false;
            currentAnimFrames.clear();
        }
    }
}
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::dictBegin(){
	dict += 1;
}
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::dictEnd(){
	dict -= 1;

	if(1 == dict){
		if(readSprites){
			lhSprites.push_back(new SpriteInfo);
			lhSprites[lhSprites.size()-1]->properties = currentSpriteInfo.properties;
			lhSprites[lhSprites.size()-1]->fixtures = currentSpriteInfo.fixtures;
            currentSpriteInfo.properties.clear();
            currentSpriteInfo.fixtures.clear();
		}
        
        if(readAnims){
            allAnims.push_back(new AnimInfo);
            allAnims[allAnims.size()-1]->properties = currentAnimInfo.properties;
            allAnims[allAnims.size()-1]->frames = currentAnimInfo.frames;
            currentAnimInfo.properties.clear();
            currentAnimInfo.frames.clear();
            readAnims = false;
            readFrames= false;
        }
	}
}
////////////////////////////////////////////////////////////////////////////////
std::string SpriteHelperLoader::valueForField(const std::string& field)
{
	std::size_t posStart = field.find_first_of(">");
	std::size_t posEnd = field.find_last_of("<");
	return field.substr(posStart+1, posEnd-posStart-1);
}
////////////////////////////////////////////////////////////////////////////////
void SpriteHelperLoader::loadSpriteHelperSceneFile(const std::string& levelFile)
{
    unsigned char* levelFileBuffer = NULL;
    unsigned long bufferSize = 0;
    std::string fullPath = CCFileUtils::fullPathFromRelativePath(levelFile.c_str());
    levelFileBuffer = CCFileUtils::getFileData(fullPath.c_str(), "r", &bufferSize);
    
	assertMessage(bufferSize > 0, "Invalid SpriteHelper file. Please add the scene file to Resource folder.");
    
    std::string filecontents((const char*) levelFileBuffer, bufferSize);
    std::stringstream infile(filecontents, stringstream::in);
    
	while(!infile.eof())
	{
		std::string line;
		getline(infile,line);
		
		if (std::string::npos != line.find("<key>")){
			key(valueForField(line).c_str());
			
		}
		else if (std::string::npos != line.find("<string>")){
			pushValue(valueForField(line).c_str());
		}
		else if (std::string::npos != line.find("<real>")){
			pushValue(valueForField(line).c_str());
		}
		else if (std::string::npos != line.find("<integer>")){
			pushValue(valueForField(line).c_str());
		}
		else if (std::string::npos != line.find("<true/>")){
			pushValue("1");
		}
		else if (std::string::npos != line.find("<false/>")){
			pushValue("0");
		}
		else if (std::string::npos != line.find("<dict>")){
			dictBegin();
		}
		else if (std::string::npos != line.find("</dict>")){
			dictEnd();
		}
		else if (std::string::npos != line.find("<array>")){
			arrayBegin();
		}
		else if (std::string::npos != line.find("</array>")){
			arrayEnd();
		}
		else if (std::string::npos != line.find("<array/>")){
			noArray();
		}
	}
        
    if (levelFileBuffer)
    {
        delete [] levelFileBuffer;
        levelFileBuffer = NULL;
    }
	
	
	readSprites = false;
	currentPolygonInfo.clear();
	currentFixtureInfo.clear();
	readFixture = false;
	currentKey = "";	
	dict = 0;
	array = 0;
    
    
    //we do this in order to be easier to find a sprite info when we want to 
    for(size_t i = 0; i < allAnims.size(); ++i)
    {
        AnimInfo* anim = allAnims[i];
        shAnims[anim->properties["UniqueName"]] = anim;
    }
    allAnims.clear();

	
	///////////////////DEBUG////////////////////////////////
	/*
	for(size_t i = 0; i < lhSprites.size(); ++i)
	{
		SpriteInfo* sprite = lhSprites[i];
		printf("-----------------SPRITE-------------------------------\n");
		std::map<std::string, std::string>::iterator It;
		for (It = sprite->properties.begin() ; It != sprite->properties.end(); It++ )
		{
			printf("key %s = %s\n", (*It).first.c_str(), (*It).second.c_str());
		}
		
        for(int j = 0; j < sprite->fixtures.size(); ++j)
        {
            std::vector<std::string> fix = sprite->fixtures[j];
            
            printf("FIXTURE\n");
            for(int k = 0; k < fix.size(); ++k)
            {
                printf("pt: %s\n", fix[k].c_str());
            }
        }
        
		printf("-----------------------------------------------------\n");
	}
    
    for(size_t i = 0; i < lhAnims.size(); ++i)
    {
        AnimInfo* anim = lhAnims[i];
        printf("-----------------ANIM INFO-------------------------------\n");
		std::map<std::string, std::string>::iterator It;
		for (It = anim->properties.begin() ; It != anim->properties.end(); It++ )
		{
			printf("key %s = %s\n", (*It).first.c_str(), (*It).second.c_str());
		}
        
        printf("Frames\n");
		for(int j = 0; j < anim->frames.size(); ++j)
        {
            std::string frame = anim->frames[j];
            
            printf("frm: %s\n", frame.c_str());
        }
        
		printf("-----------------------------------------------------\n");
    }
     */
}