//  This file was generated with SpriteHelper
//  http://www.spritehelper.org
//
//  SpriteHelperLoader.h 
//  Created by Bogdan Vladu
//  Copyright 2011 Bogdan Vladu. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  This software is provided 'as-is', without any express or implied
//  warranty.  In no event will the authors be held liable for any damages
//  arising from the use of this software.
//  Permission is granted to anyone to use this software for any purpose,
//  including commercial applications, and to alter it and redistribute it
//  freely, subject to the following restrictions:
//  The origin of this software must not be misrepresented; you must not
//  claim that you wrote the original software. If you use this software
//  in a product, an acknowledgment in the product documentation would be
//  appreciated but is not required.
//  Altered source versions must be plainly marked as such, and must not be
//  misrepresented as being the original software.
//  This notice may not be removed or altered from any source distribution.
//  By "software" the author refers to this code file and not the application 
//  that was used to generate this file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Version history
//  v1.0 First draft
//  v1.1 Added more animation methods
//  v1.2 Updated for SpriteHelper 1.6
////////////////////////////////////////////////////////////////////////////////////////////////////////



#ifndef __SPRITE_HELPER_LOADER__
#define __SPRITE_HELPER_LOADER__

#include "cocos2d.h"
#include "Box2D.h"
#include "map"

using namespace cocos2d;

//Pixel to metres ratio. Box2D uses metres as the unit for measurement.
//This ratio defines how many pixels correspond to 1 Box2D "metre"
//Box2D is optimized for objects of 1x1 metre therefore it makes sense
//to define the ratio so that your most common object type is 1x1 metre.
#define PTM_RATIO 32 //you should only keep this here so you don't change it everywhere - so you should delete it from everywhere else

typedef std::map<std::string, std::string> LHDictionary;
class SpriteInfo;
class AnimInfo;

class SpriteHelperLoader : public SelectorProtocol
{
	
public:
    
////////////////////////////////////////////////////////////////////////////////	
	/**
	 Constructor - create a new SpriteHelperLoader object\n
	 
	 @param[in] sceneFile - a string representing the SpriteHelper scene file name 
	 
	 @code 
	 SpriteHelperLoader* loader = new SpriteHelperLoader("Scene1.pshs");
	 @endcode
	 */
	SpriteHelperLoader(const std::string& levelFile);
	
////////////////////////////////////////////////////////////////////////////////
	
	virtual ~SpriteHelperLoader();
	
////////////////////////////////////////////////////////////////////////////////

    CCSprite*   spriteWithUniqueName(const std::string &name, 
                                     const CCPoint& position, 
                                     CCLayer* layer);

    b2Body*     bodyWithUniqueName(const std::string& uniqueName, 
                                   const CCPoint& position,
                                   CCLayer* layer,
                                   b2World* world);

////////////////////////////////////////////////////////////////////////////////
    
    CCSprite*   spriteInBatchWithUniqueName(const std::string &name, 
                                            const CCPoint& position, 
                                            CCLayer* layer);

    b2Body*     bodyInBatchWithUniqueName(const std::string& uniqueName, 
                                          const CCPoint& position,
                                          CCLayer* layer,
                                          b2World* world);

////////////////////////////////////////////////////////////////////////////////
    
    //the returned CCAction* should be used to stop the animation 
    //e.g: yourSprite->stopAction(returnedAction);
    //and if the animation does not have "Start At Launch" activated it should be used
    //to start the animation e.g: yourSprite->runAction(returnedAction);
    CCAction*   runAnimationWithUniqueName(const std::string& animName, 
                                           CCSprite* onSprite);

    //NOTIFICATIONS
    //NOTIFICATIONS WORKS ONLY ON ANIMATIONS THAT DONT RUN IN A LOOP
    //signature for registered method should be like this: void HelloWorld::animationEndedOnSprite(CCSprite* spr);
    //registration is done like this:  
    /*
    sh->runAnimationWithUniqueName("AnimName", sprite, callfuncND_selector(HelloWorld::animationEndedOnSprite), this);
    */
    CCAction*   runAnimationWithUniqueName(const std::string& animName, 
                                           CCSprite* onSprite,
                                           SEL_CallFuncND notifSEL,
                                           SelectorProtocol* notifObj);
    
    CCAction*   runAnimationWithUniqueName(const std::string& animName, 
                                           const b2Body* onBody);
    
    CCAction*   runAnimationWithUniqueName(const std::string& animName, 
                                           const b2Body* onBody,
                                           SEL_CallFuncND notifSEL,
                                           SelectorProtocol* notifObj);
    
////////////////////////////////////////////////////////////////////////////////
private:
	
	SpriteHelperLoader(){}
	
	std::vector<SpriteInfo*> lhSprites;
    
    std::map<std::string, AnimInfo*> shAnims;
    
    std::map<std::string, CCSpriteBatchNode*> batchInfo;
    int batchOrder;
    
	void setSpritePropertiesFromMap(CCSprite* ccsprite, LHDictionary &spriteMap);
	
	void setFixtureDefPropertiesFromMap(LHDictionary& physic,
										b2FixtureDef *shapeDef);

	b2Body* b2BodyFromMap(LHDictionary& properties,
						  std::vector<std::vector<std::string> >&fixtures,
						  CCSprite* ccsprite,
						  b2World* world);
	
    CCSprite* spriteFromMap(LHDictionary &spriteMap);
    
    LHDictionary* propertyForName(const std::string& name);
    SpriteInfo* spriteInfoForName(const std::string& name);
    
	void key(const std::string& arg);
	void pushValue(const std::string& arg);
	void arrayBegin();
	void noArray();
	void arrayEnd();
	void dictBegin();
	void dictEnd();
	std::string valueForField(const std::string& field);
	
	void loadSpriteHelperSceneFile(const std::string& levelFile);
	
	void assertMessage(bool value, const std::string message);
};

#endif
